//
//  ViewController.swift
//  FlickrSearch
//
//  Created by Michael on 13.11.14.
//  Copyright (c) 2014 Michael. All rights reserved.
//

import UIKit

class ViewController: UIViewController, UICollectionViewDelegateFlowLayout , UISearchBarDelegate, UICollectionViewDelegate, UICollectionViewDataSource {

    @IBOutlet weak var searchBar: UISearchBar!
    @IBOutlet weak var collectionView: UICollectionView!
    
    weak var currentIndexPath:NSIndexPath?
    
    var searches = Array<String>()
    var searchResults = Dictionary<String,Array<FlickrPhoto>>()
    var flickr = Flickr()
    
    override func viewDidLoad() {
        //collectionView.registerClass(UICollectionViewCell.self, forCellWithReuseIdentifier: "FlickrCell")
        
        //add long press gesture
        let longPressRecognizer = UILongPressGestureRecognizer(target: self,
                                                               action: #selector(ViewController.showMenu))
        self.collectionView?.addGestureRecognizer(longPressRecognizer)
    }

    func searchBarSearchButtonClicked(searchBar: UISearchBar) {
        flickr.searchFlickrForTerm(searchBar.text, completionBlock: {
            (searchTerm:String!, results:[AnyObject]!, error:NSError!) in
            if results != nil && results.count > 0 {
                //let index = self.searches.indexOf(searchTerm)
                if (self.searches.indexOf(searchTerm) == nil) {
                    print("Found \(results.count) photos matching: \"\(searchTerm)\"")
                    self.searches.insert(searchTerm, atIndex:0)
                    self.searchResults[searchTerm] = results as? [FlickrPhoto]
                }
                dispatch_async(dispatch_get_main_queue(), {
                    // TODO: update collection view
                    self.collectionView.reloadData()
                })
            }
            else {
                print("Error searching Flickr: \(error.localizedDescription)")
            }
        })
        searchBar.resignFirstResponder()
    }
    
    //Setting the Label
    func collectionView(collectionView: UICollectionView, viewForSupplementaryElementOfKind kind: String, atIndexPath indexPath: NSIndexPath) -> UICollectionReusableView {
        
        //Get current header
        let header = collectionView.dequeueReusableSupplementaryViewOfKind(UICollectionElementKindSectionHeader, withReuseIdentifier: "HeaderView", forIndexPath: indexPath) as! HeaderView
        
        //Set text to current search term
        header.headerLabel.text = searches[indexPath.section]
        
        //Return headerView
        return header
    }
    
    
    
    // Liefert die Anzahl von Items innerhalb einer Sektion
    func collectionView(collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        let searchTerm = searches[section]
        return searchResults[searchTerm]!.count
    }
    
    
    // Liefert die Anzahl von Sektionen
    func numberOfSectionsInCollectionView(collectionView: UICollectionView) -> Int {
        return searches.count
    }
    
    // Liefert eine wiederverwendbare bzw. neue Zelle
func collectionView(collectionView: UICollectionView, cellForItemAtIndexPath indexPath: NSIndexPath) -> UICollectionViewCell {
    let cell = collectionView.dequeueReusableCellWithReuseIdentifier("FlickrCell", forIndexPath: indexPath) as! FlickrPhotoCell
    let searchTerm = searches[indexPath.section]
    cell.photo = searchResults[searchTerm]![indexPath.item]
    return cell
}
    
    func collectionView(collectionView: UICollectionView, didSelectItemAtIndexPath indexPath: NSIndexPath) {
        // TODO: Wähle ein Item
    }
    
    func collectionView(collectionView: UICollectionView, didDeselectItemAtIndexPath indexPath: NSIndexPath) {
        // TODO: Hebe Auswahl des Items auf
    }
    
    
    // Bestimmt die Zellgröße
    /*func collectionView(collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, sizeForItemAtIndexPath indexPath: NSIndexPath) -> CGSize {
        // Bestimmen des Fotos für den entsprechenden Index
        let searchTerm = searches[indexPath.section]
        let photo = searchResults[searchTerm]![indexPath.item]
        // Berechnen Zellgröße (falls Foto noch nicht da, setze Größe auf 100x100)
        var retval = photo.thumbnail.size.width > 0 ? photo.thumbnail.size : CGSizeMake(100, 100)
        // Fügt dem Foto einen Rand hinzu
        retval.height += 35
        retval.width += 35
        return retval

    }*/
    
    // Bestimmt die Zellgröße
    func collectionView(collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, sizeForItemAtIndexPath indexPath: NSIndexPath) -> CGSize {
        // Berechnen Zellgröße
        let cellWidth = self.collectionView.bounds.size.width / 3 - 20
        return CGSizeMake(cellWidth, cellWidth)
    }
    
    func collectionView(collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, insetForSectionAtIndex section: Int) -> UIEdgeInsets {
        return UIEdgeInsetsMake(50, 10, 50, 10)
    }
    
    
    
    // Delete Section
    
    
    // override required functions
    override func canBecomeFirstResponder() -> Bool {
        return true
    }
    
    func collectionView(collectionView: UICollectionView,
                        shouldShowMenuForItemAtIndexPath indexPath: NSIndexPath) -> Bool {
        return true
    }
    
    func collectionView(collectionView: UICollectionView, canPerformAction action: Selector, forItemAtIndexPath indexPath: NSIndexPath, withSender sender: AnyObject?) -> Bool {
        return action == #selector(deleteSelectedPhoto)
    }
    
    
    // show menu
    func showMenu(gesture: UILongPressGestureRecognizer){
        if gesture.state == UIGestureRecognizerState.Began {
            // indexPath for gesture position
            let indexPath = self.collectionView?.indexPathForItemAtPoint(gesture.locationInView(self.collectionView))
            
            if indexPath != nil {
                // select cell at indexPath
                let cell = self.collectionView?.cellForItemAtIndexPath(indexPath!)
                // create menu
                let menu = UIMenuController.sharedMenuController()
                let deleteMenuItem = UIMenuItem(title: "Delete", action: #selector(deleteSelectedPhoto))
                menu.setTargetRect(CGRectMake(0, 5, 60, 80), inView: (cell?.contentView)!)
                menu.menuItems = [deleteMenuItem]
                menu.setMenuVisible(true, animated: true)
                currentIndexPath = indexPath
            }
        }
        
        
    }
    
    
    // delete function
    func deleteSelectedPhoto(){
        // path to deleted photo
        let path = currentIndexPath!
        
        // remove photo from datasource object
        searchResults[searches[path.section]]!.removeAtIndex(path.row)
        
        // remove item from collectionView
        self.collectionView.deleteItemsAtIndexPaths([path])

        // update
        self.collectionView.reloadData()
    }
    
    
    
    
    
    
    
}

